function [v_CE, dA, tau_m, F_m, F_CE] = muscle(m, phi, l_CE, A, S)
%MUSCLE calculates the muscle dynamics according to [1] and [2].
%   INPUT:
%   m: muscle parameters (struct)
%   phi: joint angles [3 by 1 rad]
%   l_CE: length of contractive element [m]
%   A: muscle activation []
%   S: muscle stimulation []
%   OUTPUT:
%   v_CE: velocity of contractive element [m/s]
%   dA: derivative of muscle activation [/s]
%   tau_m: muscle torque [3 by 1 rad]
%   F_m: muscle force [N]
%   [1] Geyer, H. and H. Herr (2010). "A muscle-reflex model that encodesl_
%   principles of legged mechanics produces human walking dynamics and
%   muscle activities." Neural Systems and Rehabilitation Engineering, IEEE
%   Transactions on 18(3): 263-273.
%   [2] Geyer, H., A. Seyfarth, et al. (2003). "Positive force feedback in 
%   bouncing gaits?" Proceedings of the Royal Society of London. Series B: 
%   Biological Sciences 270(1529): 2173.

%#codegen

%% model parameters
% fixed model parameters
N = 1.5; % eccentric force enehancement []
K = 5; % cusvature constant []
eps_ref = 0.04; % refernece strain of the series elastic element []
w = 0.56*m.l_opt; % width [m]
c = 0.05; % residual force factor
t_ecc = 0.01; % exitation contraction coupling constant [s]

% lenght properties MTU
% r = [m.r0(1); m.r0(2:3).*cos(phi(2:3)-m.phi_max(2:3))]; % effective lever arm
r = [m.r0(1); ...
    m.r0(2) .* cos(phi(2) - m.phi_max(2)); ...
    m.r0(3) .* cos(phi(3) - m.phi_max(3))];
dl_MTU = m.dir .* (...
    m.rho .* m.r0(1) .* (phi(1)-m.phi_ref(1)) + ...
    m.rho .* m.r0(2) .* (sin(phi(2) - m.phi_max(2)) - sin(m.phi_ref(2) - m.phi_max(2))) + ...
    m.rho .* m.r0(3) .* (sin(phi(3) - m.phi_max(3)) - sin(m.phi_ref(3) - m.phi_max(3)))); % deflection of the muscle

l_MTU = m.l_opt + m.l_slack + dl_MTU; % total length of the muscle tendon unit
l_SE = l_MTU - l_CE; % length of the series elastic element

%% activation dynamics
dA = (S-A)/t_ecc;

%% series elastic element
eps_SE = (l_SE-m.l_slack)/m.l_slack;

if eps_SE > 0
    f_SE = (eps_SE/eps_ref).^2;
else
    f_SE = 0;
end
F_SE = f_SE * m.F_max;

%% buffer element
l_min = m.l_opt - w;
eps_BE = w/2;
F_BE = m.F_max*(min(0,(l_CE-l_min))/(eps_BE))^2; % buffer element only exites if compressed

%% paralel element
eps_PE = w;
F_PEi = m.F_max*((max(0,l_CE-m.l_opt))/(eps_PE))^2; % paralel element only exites if tensioned

%% contractile element
% force length relationship
f_l = exp(log(c).*abs((l_CE-m.l_opt)./w).^3);

% output value based on force length relationship
f_v = (F_SE+F_BE)/(A*m.F_max*f_l+F_PEi);

% % force length relationship (NOT USED)
% if v_CE < 0
%     f_v = (m.v_max-v_CE)/(m.v_max+K*v_CE);
% else
%     f_v = N + (N+1) * (m.v_max + v_CE)/(-m.v_max+7.56*K*v_CE);
% end

% inverse force velocity relationship
if f_v <= 1
    v_CE = m.v_max.*(1-f_v)./(K*f_v+1);
elseif (f_v > 1) && (f_v <= N)
    f_v1 = (f_v-N)/(N-1);
    v_CE = m.v_max.*(1+f_v1)./(7.56*K*f_v1-1);
else
%     v_CE = -m.v_max; 
    v_CE = (f_v-N)*0.01-m.v_max;
end

F_CE = A * f_v * f_l * m.F_max;

% muscle force and joint torque
F_m = F_SE; % muscle force
tau_m = r .* (F_m .* m.dir); % joint torque

end

