classdef multibody < handle
    %MULTIBODY Summary of this class goes here
    %   Detailed explanation goes here
    
    properties
        parts
        worlds
        joints
        masses
        points
        constraints
        controllers
        simevents
        nq % number of generalized coordinates
        ns % number of additional states (used by controller)
        q_dep % dependent cordinates
        x0
        dx0
        s0
        rec0
        nrec % number of recorded states (used by controllers)
        fig % figure for the gui
        axs % axes in which the model is plot
        simt % time vector of simulation results
        simx % position matrix of simulation results
        simrec % recorded states in the simulation
        simcons
        prefix = '' % prefix for the files to allow models to run at once
        t = [0 1]
        timer % timer for real time animation
        dt = 0.0005 % sample time
    end
    
    methods
        function mdl = multibody
            mdl.parts{1} = world;
        end
        
        function mdl = makepart(mdl, newpart)
            if isempty(newpart.name)
                newpart.name = ['p_' num2str(length(mdl.parts) + 1)];
            end
            if ~isempty(newpart.parentname)
                newpart.parent = mdl.findpart(newpart.parentname);
            end
            if ~isempty(newpart.childname)
                newpart.child = mdl.findpart(newpart.childname);
            end
            mdl.parts{end+1} = newpart;
        end
        
        function mdl = makecontroller(mdl, controller)
            if isempty(mdl.controllers);
                mdl.controllers{1} = controller;
            else
                mdl.controllers{end+1} = controller;
            end
        end
        
        function mdl = preprocess(mdl)
            mdl = preprocessm(mdl);
        end
        
        function makemex(mdl)
            mdl.initmatrix;
            mexcfg = coder.config('mex');
            % mexcfg.DynamicMemoryAllocation = 'AllVariableSizeArrays';
            nin = (mdl.nq + sum(~mdl.q_dep) + mdl.ns);
            xsample = zeros(nin,1);
            codegen -config mexcfg rk4fixed -args {coder.typeof(double(0),[1 40001],1), coder.typeof(double(0),[nin 40001],1), coder.typeof(double(0),[mdl.nrec 40001],1), mdl.simcons, 0, 0, xsample}
        end
        
        function makefigure(mdl)
            mdl.fig = figure();
            mdl.axs = axes();
            hold on; axis equal;
            mdl.updatefigure
        end
        
        function updatefigure(mdl)
            cla(mdl.axs)
            for ipart = [mdl.masses mdl.joints mdl.points]
                mdl.parts{ipart}.makefill(mdl.axs);
            end
        end
        
        function setfigure(mdl, x, myplotfun)
            [Tj, Tm, Tp] = ...
                Tmatrices(x, zeros(size(x)), mdl.simcons.p_in);
            for ijoint = 1:length(mdl.joints);
                mdl.parts{mdl.joints(ijoint)}.setfill(Tj(ijoint * 3 + [-2 -1 0]))
            end
            for imass = 1:length(mdl.masses);
                mdl.parts{mdl.masses(imass)}.setfill(Tm(imass * 3 + [-2 -1 0]))
            end
            for ipoint = 1:length(mdl.points);
                mdl.parts{mdl.points(ipoint)}.setfill(Tp(ipoint * 3 + [-2 -1 0]))
            end
            if ~isempty(myplotfun)
                myplotfun();
            end
        end
        
        function resample(mdl, newsampletime)
            % intended use is to save space when saving the model to disk
            
            % resample the data
            t_new = mdl.simt(1):newsampletime:mdl.simt(end);
            mdl.simx = interp1(mdl.simt, mdl.simx', t_new)';
            mdl.simrec = interp1(mdl.simt, mdl.simrec', t_new)';
            
            mdl.rec0 = interp1(mdl.t(1):mdl.dt:mdl.t(2), mdl.rec0(:,1:end)', mdl.t(1):newsampletime:mdl.t(2))';
            mdl.simt = t_new;
            mdl.dt = newsampletime;
            
            % clear the simulation constants
            mdl.simcons = [];
        end
        
        function simulate(mdl, mex, resume)
            % simulation
            % mex = 1: use mex files
            % mex = 0: use script files
            % resume = 1: resume the simulation
            % resume = 0: start from 1
            % to resume the simulation the model should not be changed in
            % between.
            
            mdl.initmatrix;
            t_in = mdl.t(1):mdl.dt:mdl.t(2);
            
            N = length(t_in);
            xinit = [mdl.x0; mdl.dx0(mdl.q_dep == 0); mdl.s0];
            x = zeros(length(xinit), N);
            
            if resume
                Nstart = size(mdl.simx, 2);
                x(:, 1:Nstart) = mdl.simx;
                mdl.rec0(:, 1:Nstart) = mdl.simrec;
            else
                % calculate the first frame
                [~, mdl.rec0(:, 1)] = equations_of_motion(t_in(1), xinit, mdl.t, x, mdl.rec0, 1, mdl.dt, mdl.simcons);
                x(:,1) = xinit;
                Nstart = 1;
            end
            
            if mex
                [mdl.simt mdl.simx mdl.simrec Nend] = rk4fixed_mex(...
                    t_in, x, mdl.rec0, mdl.simcons, mdl.dt, Nstart, xinit);
            else
                [mdl.simt mdl.simx mdl.simrec Nend] = rk4fixed(...
                    t_in, x, mdl.rec0, mdl.simcons, mdl.dt, Nstart, xinit);
            end
            mdl.simt = mdl.simt(1:Nend-1);
            mdl.simx = mdl.simx(:, 1:Nend-1);
            mdl.simrec = mdl.simrec(:, 1:Nend-1);
        end
        
        function animate(mdl, varargin)
            if nargin > 1
                myplotfun = varargin{1};
            else
                myplotfun = [];
            end
            
            animator(mdl.fig, @(t) mdl.setfigure(mdl.simx(:, t), myplotfun), mdl.simt, ...
                'Speed', 1, ...
                'TimeLines', true, ...
                'StartPlay', false);
        end
        
        function rtanimate(mdl, varargin)
            if nargin > 1
                myplotfun = varargin{1};
            else
                myplotfun = [];
            end
            
            timestep = 0.001; % the simulation time step
            tmulti = 40; % time step multiplier
            remember = 100;
            slowmotion = 2;
            
            mdl.t = [0 timestep * tmulti];
            mdl.simulate(1,0)
            
            mdl.timer = timer('ExecutionMode','fixedDelay',...  % fire at a fixed rate
                'Period',timestep * tmulti * slowmotion,...    % every x seconds
                'BusyMode', 'error',...
                'TimerFcn',@timer_fcn);  % run this function
            
            start(mdl.timer); % Start timer, which runs on its own until it ends
            function timer_fcn(obj, ~)
                n = get(obj,'TasksExecuted'); % number of tasks executed
                if n < remember
                    mdl.t(2) = mdl.t(2) + timestep * tmulti;
                    mdl.simulate(1,1);
                else
                    mdl.simt = mdl.simt((tmulti+1):end);
                    mdl.simx = mdl.simx(:, (tmulti+1):end);
                    mdl.simrec = mdl.simrec(:, (tmulti+1):end);
                    mdl.t = mdl.t + timestep * tmulti;
                    mdl.simulate(1,1);
                end
                mdl.setfigure(mdl.simx(:, end), myplotfun)
            end
        end
        
        function mdl = initmatrix(mdl)
            jmp = [mdl.joints mdl.masses mdl.points];
            mdl.simcons.p_in = zeros(length(jmp), 2);
            for ipart = 1:length(jmp)
                mdl.simcons.p_in(ipart,:) = mdl.parts{jmp(ipart)}.getinitpos;
            end
            
            mdl.simcons.c_in = zeros(length(mdl.constraints), 4);
            for icons = 1:length(mdl.constraints)
                mdl.simcons.c_in(icons,:) = mdl.parts{mdl.constraints(icons)}.getinitposcons;
            end
            
            %% initial positions and velocities
            mdl.x0 = [];
            mdl.dx0 = [];
            for i = 1:length(mdl.joints)
                ijoint = mdl.joints(i);
                if ~isempty(mdl.parts{ijoint}.q0)
                    mdl.x0 = [mdl.x0; mdl.parts{ijoint}.q0];
                else
                    mdl.x0 = [mdl.x0; zeros(length(mdl.parts{ijoint}.q), 1)];
                end
                if ~isempty(mdl.parts{ijoint}.dq0)
                    mdl.dx0 = [mdl.dx0; mdl.parts{ijoint}.dq0];
                else
                    mdl.dx0 = [mdl.dx0; zeros(length(mdl.parts{ijoint}.q), 1)];
                end
            end
            
            %% solve the constraints
            % if ~isempty(mdl.constraints)
            %     mdl = solveconstraints(mdl); % solve the initial conditions
            % end
            
            %% the mass (M) matrix
            for imass = 1:length(mdl.masses)
                mdl.simcons.M((imass * 3 - 2) : (imass * 3), (imass * 3 - 2) : (imass * 3)) = ...
                    diag(mdl.parts{mdl.masses(imass)}.m);
            end
            
            %% initialize the controllers and set s0
            mdl.s0 = [];
            mdl.rec0 = [];
            for icontroller = 1:length(mdl.controllers)
                range = mdl.t(1):mdl.dt:mdl.t(2);
                mdl.controllers{icontroller}.initsim(mdl);
                eval(['mdl.simcons.control.c' num2str(icontroller) ' = mdl.controllers{icontroller}.cons;']);
                mdl.s0 = [mdl.s0; mdl.controllers{icontroller}.s0];
                if ~isempty(mdl.controllers{icontroller}.rec0)
                    mdl.rec0 = [mdl.rec0; mdl.controllers{icontroller}.rec0];
                else
                    mdl.rec0 = [mdl.rec0; zeros(mdl.controllers{icontroller}.rec, length(range))];
                end
            end
        end
        
        function setshape(mdl)
            %% set the shapes of the parts
            for ipart = [mdl.masses mdl.joints mdl.points]
                mdl.parts{ipart}.setshape;
            end
        end
        
        function [fpart ipart] = findpart(mdl, partname)
            fpart = [];
            for ipart = 1:length(mdl.parts)
                if strcmp(partname, mdl.parts{ipart}.name)
                    fpart = mdl.parts{ipart};
                    break
                end
            end
        end
        
        function fcontrol = findcontrol(mdl, controlname)
            fcontrol = [];
            for icontrol = 1:length(mdl.controllers)
                if strcmp(controlname, mdl.controllers{icontrol}.name)
                    fcontrol = mdl.controllers{icontrol};
                    break
                end
            end
        end
    end
    
    methods(Static)
        function delarray = deletefromarray(delarray, delobject)
            delindex = [];
            for i = length(delarray)
                if delarray{i} ~= delobject
                    delindex = [delindex i];
                end
            end
            delarray = delarray(delindex);
        end
    end
    
end

