function [t x rec iend] = rk4fixed(t, x, rec, simcons, h, Nstart, xsample)
% Matlab implementation of a fixed-step RK4 algorithm.
% Written by R.G. Longoria 950129.
% This version was revised July 1998 to be compatible with
% the Matlab ODE suite.
% Usage: >> [T,Y] = rk4fixed('Fcn',[T0 Tf],X0,N);
% Inputs to program:
% X0 = initial value(s) of the dependent variable.
%      Example: X0=[1;1];
% Tspan = vector, [T0 Tf]
% T0 = starting value of the independent variable (e.g. time)
% Tf = ending value of the independent variable
% h = time step
% Fcn = the name of the function that defines the ODEs, f(t,x).
%     Note: User needs to define this function in an m-file:
%           function yprime=Fcn(T,X)
% Computed in rk4fixed:
% h = fixed step size = (Tspan(2)-Tspan(1))/N
% Output from rk4fixed:
% Two arrays: X and T over the time period specified.
%
% Here is an example of the use of this program from\\
% the command line:
%      [T,X]=rk4fixed(`myfunction',[0 0.5],[0.1;0.0],500);
%      This integrates 2 eqs specified in `myfunction' from
%      time 0 to 0.5 with steps of h=(0.5-0)/500.  The initial
%      conditions are given by [0.1;0.0].
%

%#codegen

iend = 0;
halfh = 0.5*h;
N = length(t);

% i_end = Nstart;

td = t(Nstart);
xd = zeros(size(xsample));
xd(:) = x(:,Nstart);

for i = (Nstart+1):N
    [RK1 rec(:, i) events] = equations_of_motion(td, xd, t, x, rec, i, h, simcons);
    iend = i;
    if events
        break
    end
    
    Thalf = td + halfh;
    Xtemp = xd + halfh*RK1;
    RK2 = equations_of_motion(Thalf, Xtemp, t, x, rec, i, h, simcons);
    
    Xtemp = xd + halfh*RK2;
    RK3 = equations_of_motion(Thalf, Xtemp, t, x, rec, i, h, simcons);
    
    Tfull = td + h;
    Xtemp = xd + h*RK3;
    RK4 = equations_of_motion(Tfull, Xtemp, t, x, rec, i, h, simcons);
    
    x(:,i) = xd + h*(RK1+2.0*(RK2+RK3)+RK4)/6;
    
    xd(:) = x(:,i);
    td = t(i);  
end

