function averagestep(mdl, varargin)
%AVERAGESTEP Show the average step data for a simulation with a human
%walker
%IN:
%   mdl: the model and the simulation results [object]
%   plottype: description of the disered plots [string], more then one
%   plottype is possible, give them as additional arguments
%
% available plot types:
%   activations - plots the activation of the muscles
%   muscle force
%   joint torque
%   joint torque area
%   joint angles
%
% created by: Wietse van Dijk, Delft University of Technology (2013)

mdl.initmatrix;

% colormap
c = [0 166 214; ...
    161 0 88; ...
    173 198 16; ...
    0 43 96; ...
    236 127 44]/255;

% finds the steps based on contacts of the foot
steps = [];
for i = 2:length(mdl.simt)-(0.1/mdl.dt)
    if any(mdl.simrec(mdl.findcontrol('contactforce').recid([2 5]),i)) ...
            && any(mdl.simrec(mdl.findcontrol('contactforce').recid([2 5]),i+(0.1/mdl.dt))) ...
            && ~any(mdl.simrec(mdl.findcontrol('contactforce').recid([2 5]),i-1))
        steps = [steps i];
    end
end
selectedSteps = steps(end-5:end);

for iArg = 1:(nargin - 1)
    figure('name', varargin{iArg});
    switch varargin{iArg}
        case 'activations'
            %% muscle activations plot 
            reflex = mdl.findcontrol('reflex'); % select the reflex controller
            sid1 = length([mdl.x0; mdl.dx0(mdl.q_dep == 0)]);
            activations = mdl.simx(sid1 + reflex.sid(1:14),:);
                        
            for i = 1:7; % loop the muscles
                subplot(2, 4, i);
                title(reflex.cons.m(i).name); % name of the muscle as plot title
                xlabel('stride [%]');
                ylabel('activation []')
                
                normEMG = normstep(activations(i,:), selectedSteps)';
                confplot(1:100, mean(normEMG),std(normEMG),std(normEMG),'b',1,0.5,'-')
            end
        case 'muscle force'
            %% muscle force plot         
            reflex = mdl.findcontrol('reflex'); % select the reflex controller
            
            for i = 1:7
                subplot(2, 4, i)
                title(reflex.cons.m(i).name); % name of the muscle as plot title
                xlabel('stride [%]');
                ylabel('muscle force [N]')
                
                normforce = normstep(mdl.simrec(reflex.recid(i),:), selectedSteps)';
                confplot(1:100,mean(normforce),std(normforce),std(normforce),'b',1,0.5,'-');
                
                y = ylim; ylim([-10, y(2)]); % set lower limit y-axis
            end
        case {'joint torque', 'joint torque area', 'bar'} % plots that use joint torques
            % find the mass of the walker
            diagM = diag(mdl.simcons.M);
            M = sum(diagM(1:3:end));
            
            titles = {'hip', 'knee', 'ankle'}; % titles for the subplots
            torquecontrollers = {'reflex', 'ligaments', 'ff_gaitpattern', 'ferris_support'}; % controllers that generate joint torques
            torquecontrollerNames = {'muscle-reflex model', 'ligament torques', 'ankle torque feedforward (AFF)', 'soleus activation feedback (SFB)'}; % names for the torque controllers
            
            ntor = 0;
            cnames = {};
            for iC = 1:length(torquecontrollers)
                if ~isempty(mdl.findcontrol(torquecontrollers{iC}))
                    ntor = ntor + 1;
                    tor(ntor, :, :) = mdl.simrec(mdl.findcontrol(torquecontrollers{iC}).recid(end-5:end),:)/M;
                    cnames{end+1} = torquecontrollerNames{iC};
                end
            end
            
            switch varargin{iArg}                    
                case 'joint torque'
                    %% area plot different controllers
                    for i = 1:3
                        subplot(1, 3, i); hold on
                        title(titles(i))
                        xlabel('stride [%]');
                        ylabel('normalized torque [Nm/kg]')
                        for iC = 1:ntor
                            normTorque  = normstep(squeeze(tor(iC,i,:)), selectedSteps)';
                            confplot(1:100,mean(normTorque), std(normTorque), std(normTorque),c(iC, :),1,0.5,'-');
                        end
                    end
                    legend(cnames, 'Location', 'SouthEast')
                case 'joint torque area'
                    %% area plot different controllers
                    for i = 1:3
                        subplot(1, 3, i);
                        title(titles(i))
                        xlabel('stride [%]');
                        ylabel('normalized torque [Nm/kg]')
                        for iC = 1:ntor
                            t(:,iC) = mean(normstep(squeeze(tor(iC,i,:)), selectedSteps)');
                        end
                        
                        a = area(1:100, t);
                        for iPatch = 1:length(a); % loop the patches of the area plot
                            set(get(a(iPatch), 'Children'), 'FaceAlpha', 0.7); % set each patch transparent
                        end
                    end
                    legend(cnames)
            end
        case 'joint angles'
            part = {'lefthip', 'leftknee', 'leftankle'};
            titles = {'hip', 'knee', 'ankle'}; % titles for the subplots
            
            for ipart = 1:3
                subplot(1,3,ipart);
                title(titles{ipart});
                xlabel('stride [%]');
                ylabel('joint angle [rad]')
                
                partq = mdl.findpart(part{ipart}).qid;
                parentq = mdl.findpart(part{ipart}).parent.qid(end);
                
                normAngle = normstep(mdl.simx(partq, :) - mdl.simx(parentq, :), selectedSteps)';
                confplot(1:100,mean(normAngle),std(normAngle),std(normAngle),'b',1,0.5,'-')  
            end
        otherwise
            disp('this type of figure is not recognized')
    end
end


