function mdl = runpso_anklesupport(newmex, modeltype)
%RUNPSO_REFLEX_ANKLEFF Runs a particle swarm optimization for a model with
%feed forward ankle actuation. The ammount of support is increased in X
%steps.

% initialize the random number generator
rng(int32(sum(clock) * 100))

nSteps = 8; % number of steps (every step the ankle support is increased)
if strcmp(modeltype, 'nosupport')
    nSteps = 4;
end

if newmex % only make a new mex file is required by user
    % make a model and save it
    makews(1, modeltype); % make a workspace
    load workspace % load the workspace
    
    % place the model on the floor
    [~, ~, Tp] = Tmatrices(mdl.x0, mdl.dx0, mdl.simcons.p_in);
    mdl.x0(2) = -min(Tp(2:3:end));
    
    % run and save the model
    mdl.t = [0 10];
    mdl.simulate(1, 0);
    mdl.findcontrol('reflex').cons.maxnoise = 0.02;
    save(['generated' filesep 'workspace'], 'mdl');
end

% define a file name
c = clock;
savename = fullfile('results', ['anklesupport_' modeltype '_' date '_' num2str(c(4)) '-' num2str(c(5)) '-' num2str(round(c(6))) '_step']);

opt = initoptparam;
p(1) = runpso(opt, 0, modeltype, [savename '0']);

for irun = 1:nSteps-1
    bestpos = p(irun).overalbestpos(p(irun).iterations);
    % set the new optimization within +/- 15% of the previous best
    for iopt = 1:length(opt)
        opt(iopt).initbound(1) = max(opt(iopt).bound(1), bestpos(iopt) - 0.15 * (opt(iopt).bound(2)-opt(iopt).bound(1)));
        opt(iopt).initbound(2) = min(opt(iopt).bound(2), bestpos(iopt) + 0.15 * (opt(iopt).bound(2)-opt(iopt).bound(1)));
    end
    close all;
    p(irun + 1) = runpso(opt, irun, modeltype, [savename num2str(irun)]);
end
end

function [errcode, fitnessout, stageout, mdl] = objectivemultirun(x, opt, irun, modeltype, nRuns, minmax)

fitnessoutMulti = zeros(nRuns,1); 
stageoutMulti = zeros(nRuns,1);

for i = 1:nRuns
    [errcode, fitnessoutMulti(i), stageoutMulti(i), mdl] = objectivefunc(x, opt, irun, modeltype);
end

if strcmp(minmax, 'min')
    stageout = min(stageoutMulti);
    fitnessout = min(fitnessoutMulti(stageoutMulti == stageout));
else
    stageout = max(stageoutMulti);
    fitnessout = max(fitnessoutMulti(stageoutMulti == stageout));
end
end

function [errcode, fitnessout, stageout, mdl] = objectivefunc(x, opt, irun, modeltype)
m = load('workspace');
mdl = m.mdl;

errcode = 0;

for iopt = 1:length(opt)
    switch opt(iopt).type
        case 'leanangle'
            mdl.findpart('origin').q0 = [0; 0; x(iopt)];
            mdl.findpart('leftknee').q0 = x(iopt);
        case 'legangle'
            legangle1 = x(iopt);
            legangle2 = x(iopt) + (5/180*pi);
            mdl.findpart('righthip').q0 = legangle1;
            mdl.findpart('rightknee').q0 = legangle2;
            mdl.findpart('rightankle').q0 = legangle2;
        case 'swingspeed'
            mdl.findpart('righthip').dq0 = x(iopt);
            mdl.findpart('rightknee').dq0 = x(iopt);
            mdl.findpart('rightankle').dq0 = x(iopt);
        otherwise
            eval([opt(iopt).type ' = ' num2str(x(iopt)) ';']);
    end   
end

%% set the amount of support
switch modeltype
    case 'ffgaitpatternankle'
        mdl.findcontrol('ff_gaitpattern').cons.weight = irun * 0.06 * 80;
        mdl.findcontrol('ff_gaitpattern').cons.on = 1;
    case 'ferris_support'
        mdl.findcontrol('ferris_support').cons.gain = -irun * 30;
        mdl.findcontrol('ferris_support').cons.on = 1;
    case 'nosupport'
        % nothing needs to be changed
end

%% add noise to the muscle model
minfreq = 5;
maxfreq = 10;
maxt = 20;
mdl.findcontrol('reflex').cons.noiseX = [zeros(14,1) cumsum(rand(14, maxt*maxfreq)*(1/minfreq-1/maxfreq)+1/maxfreq, 2)];
mdl.findcontrol('reflex').cons.noiseY = rand(size(mdl.findcontrol('reflex').cons.noiseX));

%% run the model
mdl.t = [0 20];
mdl.simulate(1, 1);
mdl.resample(0.005);
mdl.initmatrix;

%% calculate fitness
f = walkcostfunction(mdl);
fitness = []; stage = [];

% model doesn't fall
fitness(end + 1) = f.time; 
stage(end + 1) = fitness(end) > mdl.t(2)-0.01;
% model has roughly the correct speed
fitness(end + 1) = -abs(f.speed - 1.2); 
stage(end + 1) = fitness(end) > -0.5;
% model has a uniform steplength
fitness(end + 1) = -f.steplengthstd;
stage(end + 1) = fitness(end) > -0.05;
% finetuning of the speed
fitness(end + 1) = -abs(f.speed - 1.2);
stage(end + 1) = fitness(end) > -0.1;
% optimize for torque
fitness(end + 1) = -(f.musclework * f.activation * f.ligamentstor);
% fitness(end + 1) = -(f.muscleenergy * f.ligamentstor^0.5);

stageout = find(~stage, 1) - 1; 
if isempty(stageout)
    stageout = length(stage);
end

fitnessout = fitness(stageout + 1);
end

function opt = initoptparam
% set the parameters to be optimized
m = load('workspace');
mdl = m.mdl;
opt = [];
opt(end + 1).type = 'mdl.findcontrol(''reflex'').cons.phi_knee_off'; opt(end).bound = [-60 0] / 180 * pi; opt(end).initbound = [-12 -0] / 180 * pi; % 1
opt(end + 1).type = 'mdl.findcontrol(''reflex'').cons.k_phi'; opt(end).bound = [0 10]; opt(end).initbound = [0 3]; % 2
opt(end + 1).type = 'mdl.findcontrol(''reflex'').cons.dS'; opt(end).bound = [.05 1]; opt(end).initbound = [.1 .3]; % 3
opt(end + 1).type = 'mdl.findcontrol(''reflex'').cons.theta_ref'; opt(end).bound = [0.017 0.11]; opt(end).initbound = [0.08 0.105]; % 4
opt(end + 1).type = 'mdl.findcontrol(''reflex'').cons.k_p'; opt(end).bound = [0.5 10]; opt(end).initbound = [1 3]; % was 1.78 22 % 5
opt(end + 1).type = 'mdl.findcontrol(''reflex'').cons.k_d'; opt(end).bound = [.1 0.75]; opt(end).initbound = [.1 .25]; % was .2 .3 % 6
opt(end + 1).type = 'mdl.findcontrol(''reflex'').cons.k_bw'; opt(end).bound = [.05 5]; opt(end).initbound = [.5 1.5]; % 7
opt(end + 1).type = 'mdl.findcontrol(''reflex'').cons.k_lean'; opt(end).bound = [1 5.7]; opt(end).initbound = [1 1.6]; % 8
opt(end + 1).type = 'mdl.findcontrol(''reflex'').cons.S_0_bal'; opt(end).bound = [0.01 0.32]; opt(end).initbound = [0.02 0.08]; % 9

opt(end + 1).type = 'mdl.findcontrol(''reflex'').cons.m(1).G'; opt(end).bound = [0.8 2.17] / mdl.findcontrol('reflex').cons.m(1).F_max; opt(end).initbound = [.8 1.1] / mdl.findcontrol('reflex').cons.m(1).F_max; % 10

opt(end + 1).type = 'mdl.findcontrol(''reflex'').cons.m(2).G'; opt(end).bound = [0.55 3.2] / mdl.findcontrol('reflex').cons.m(2).l_opt; opt(end).initbound = [1 1.5] / mdl.findcontrol('reflex').cons.m(2).l_opt; % muscle reflex gain 11
opt(end + 1).type = 'mdl.findcontrol(''reflex'').cons.m(2).l_off'; opt(end).bound = [0.59 0.8] * mdl.findcontrol('reflex').cons.m(2).l_opt; opt(end).initbound = [0.63 0.7] * mdl.findcontrol('reflex').cons.m(2).l_opt;% length offset 12
opt(end + 1).type = 'mdl.findcontrol(''reflex'').cons.m(2).G2'; opt(end).bound = [0 5] / mdl.findcontrol('reflex').cons.m(1).F_max; opt(end).initbound = [.6 1.6] / mdl.findcontrol('reflex').cons.m(1).F_max;% muscle reflex gain soleus - tibialis anterior coupling 13

opt(end + 1).type = 'mdl.findcontrol(''reflex'').cons.m(3).G'; opt(end).bound = [0 10] / mdl.findcontrol('reflex').cons.m(3).F_max; opt(end).initbound = [.8 3] / mdl.findcontrol('reflex').cons.m(3).F_max; % 14

opt(end + 1).type = 'mdl.findcontrol(''reflex'').cons.m(4).G'; opt(end).bound = [0 10] / mdl.findcontrol('reflex').cons.m(4).F_max; opt(end).initbound = [.5 2] / mdl.findcontrol('reflex').cons.m(4).F_max; % muscle reflex gain 15
opt(end + 1).type = 'mdl.findcontrol(''reflex'').cons.m(4).S_0'; opt(end).bound = [0.047 0.71]; opt(end).initbound = [0.05 0.15]; % initial stimulation % in model 0.08 in artikel 0.09 16

opt(end + 1).type = 'mdl.findcontrol(''reflex'').cons.m(5).G'; opt(end).bound = [0 0.67] / mdl.findcontrol('reflex').cons.m(5).F_max; opt(end).initbound = [0.5 0.67] / mdl.findcontrol('reflex').cons.m(5).F_max; % 17
opt(end + 1).type = 'mdl.findcontrol(''reflex'').cons.m(5).l_off'; opt(end).bound = [0.6 5] * mdl.findcontrol('reflex').cons.m(5).l_opt; opt(end).initbound = [0.6 0.9] * mdl.findcontrol('reflex').cons.m(5).l_opt; % 18

opt(end + 1).type = 'mdl.findcontrol(''reflex'').cons.m(6).G'; opt(end).bound = [0 0.55] / mdl.findcontrol('reflex').cons.m(6).F_max; opt(end).initbound = [0.25 0.45] / mdl.findcontrol('reflex').cons.m(6).F_max; % 19

opt(end + 1).type = 'mdl.findcontrol(''reflex'').cons.m(7).G'; opt(end).bound = [.17 3] / mdl.findcontrol('reflex').cons.m(7).l_opt; opt(end).initbound = [.5 1] / mdl.findcontrol('reflex').cons.m(7).l_opt; % 20
opt(end + 1).type = 'mdl.findcontrol(''reflex'').cons.m(7).G2'; opt(end).bound = [0 20] / mdl.findcontrol('reflex').cons.m(5).l_opt; opt(end).initbound = [2 8] / mdl.findcontrol('reflex').cons.m(5).l_opt; % 21
end

function p = runpso(opt, irun, modeltype, savename)

xbound = zeros(length(opt),2);
xinitbound = zeros(length(opt),2);

for i = 1:length(opt)
    xbound(i,:) = opt(i).bound;
    xinitbound(i,:) = opt(i).initbound;
end

p = pso('posmin', xbound(:,1)', ...
    'posmax', xbound(:,2)', ...
    'initposmin', xinitbound(:,1)', ...
    'initposmax', xinitbound(:,2)', ...
    'costfunc', @(x) objectivemultirun(x, opt, irun, modeltype, 3, 'min'), ...
    'iterations', 50, ... 
    'popsize', 40, ... 
    'initsize', 80, ...
    'localsize', 2, ...
    'socialsize', 5, ... 
    'output', {'file', 'text'}); %, 'graphic'});

if irun == 0
    p.iterations = 75;
    p.popsize = 60;
    p.initsize = 120;
    p.socialsize = 7;
end

p.savefile = savename;
p.run;

[~, ~, ~, mdl] = objectivemultirun(p.overalbestpos(p.iterations), opt, irun, modeltype, 10, 'max');
% mdl.resample(0.005); % resample the model to save space
mdl.findcontrol('reflex').rec0 = [];
save([savename '_mdl'], 'mdl')

end
