classdef pso < handle
    %PSO Summary of this class goes here
    %   Detailed explanation goes here
    
    properties
        %% Settings for Particle Swarm Optimisation
        popsize = 20 % number of particles
        initsize = [] % number of particles to pick the initial population from
        localsize = 5 % number of particles in the local group
        socialsize = 2 % number of neighbours on one side
        iterations = 50 % number of iterations
        nvars % number of variables
        crand = 0; % random speed dependence
        cpart = 1; % speed dependence particle best
        clocal = 0; % speed dependence local best
        csocial = 1; % speed dependance on the social best
        coveral = 0; % speed dependence overal best
        posmin % upper bound of variables
        posmax % lower bound of variables
        initposmin % upper bound of initial variables
        initposmax % lower bound of initial variables
        velmin = -.015; % relative minimal particle velocity
        velmax = .015; % relative maximal particle velocity
        maxfailupdate = 4; % maximal number of times the algorithm tries to find a feasible solution
        savefile % name of the save file
        costfunc % cost function to be minimized
        fig % figure
        axs % axes
        sca % scatter of the latest particles
        plt % plot with the trace of the particles
        output = {'text'} % output options array of output options ('text', 'file', and/or 'graphic')
        fitness % fitness of the particles
        stage % stage of the particles
        pos % position of the particles
        vel % velocity of the particles
        relpos % matrix that contains the distance between the particles
    end
    
    methods
        function p = pso(varargin)
            % constructor
            % create a name for the savefile
            c = clock;
            p.savefile = fullfile('results', ['PSO_' date '_' num2str(c(4)) '-' num2str(c(5)) '-' num2str(round(c(6)))]);
            % set the input arguments
            p.set(varargin{:});
        end
        
        function p = set(p, varargin)
            % SET assigns values to the class properties
            for iargin = 1:2:nargin-1
                eval(['p.' varargin{iargin} ' = varargin{iargin + 1};']);
            end
        end
        
        function p = run(p)
            p.init;
            vmin = p.velmin*(p.posmax-p.posmin);
            vmax = p.velmax*(p.posmax-p.posmin);
            for iter = 2:p.iterations % loop the iterations
                p.relpos = p.calcrelpos(iter-1);
                
                for ipart = 1:p.popsize
                    ipos(ipart,:) = squeeze(p.pos(iter-1, ipart,:));
                    ivel(ipart,:) = squeeze(p.vel(iter-1, ipart,:));
                    ibpart(ipart,:) = p.bestpos(iter-1, ipart);
                    iblocal(ipart,:) = p.localbestpos(iter-1, ipart);
                    isocial(ipart,:) = p.socialbestpos(iter-1, ipart);
                    iboveral = squeeze(p.overalbestpos(iter-1));
                end
                
                parfor ipart = 1:p.popsize
                    errcode = 1;
                    failupdate = 0;
                    while errcode > 0,  % update vel en pos until feasible solution is found
                        
                        errcode = 1;
                        failupdate = 0;
                        
                        veldum = ...
                            0.95 * ivel(ipart, :) ...
                            + p.cpart * rand * (ibpart(ipart,:) - ipos(ipart,:)) ...
                            + p.clocal * rand * (iblocal(ipart, :) - ipos(ipart,:)) ...
                            + p.csocial * rand * (isocial(ipart, :) - ipos(ipart,:)) ...
                            + p.coveral * rand * (iboveral' - ipos(ipart,:));
                        % check for maximal velocities
                        veldum = min(vmax, max(vmin, veldum));
                        % check for maximal velocities
                        veldum = min(vmax, max(vmin, veldum));
                        % update particle position
                        posdum = squeeze(p.pos(iter-1, ipart,:))' + veldum;
                        % check for postion bounds
                        posdum = min(p.posmax, max(p.posmin, posdum));
                        veldum(posdum == p.posmax) = 0; % set velocity to zero when on upper bound
                        veldum(posdum == p.posmin) = 0; % set velocity to zero when on lower bound
                        [errcode, fitnessnew(ipart) stagenew(ipart)] = p.costfunc(posdum); % evaluate objective function
                                               
                        if errcode > 0, % update particle again
                            failupdate = failupdate + 1;
                            disp([num2str(failupdate) ': no feasable solution found, retry update of particle: ' num2str(ipart)]);
                            if failupdate == p.maxfailupdate
                                errcode = -1;
                            end
                        end
                        
                        velnew(ipart, :) = veldum;
                        posnew(ipart, :) = posdum;
                        
                        if errcode == -1, % If no feasible solution is found within maximal number of attempts (set by Maxfailupdate) Particle will be initiatlised
                            disp(['Particle ' num2str(ipart) ' will be re initialised']);
                            [posnew(ipart,:) ...
                                velnew(ipart,:) ...
                                fitnessnew(ipart) ...
                                stagenew(ipart)] = ...
                                p.initparticle;
                        end
                    end
                end
                                
                p.vel(iter,:,:) = velnew;
                p.pos(iter,:,:) = posnew;
                p.fitness(iter,:) = fitnessnew;
                p.stage(iter, :) = stagenew;
                    
                % decrease interial factor when object function decrease
                % (not implemented at this moment)
                
                % save and output data of this iteration
                for iout = p.output
                    switch iout{:}
                        case 'file'
                            possave = p.pos;
                            velsave = p.vel;
                            fitsave = p.fitness;
                            save(p.savefile, 'p', 'possave', 'velsave', 'fitsave');
                        case 'graphic'
                            p.updateplot(iter);
                        case 'text'
                            [~, bofit, bostage] = p.overalbestpos(iter) ;
                            % disp(['iteration: ' num2str(iter) ' best overal fitness: ' num2str(bofit) ' average fitness last iteration: ' num2str(mean(fitnessnew))]);
                            mstage = max(p.stage(iter, :));
                            stages = zeros(1, mstage + 1);
                            for i = 1 : (mstage + 1)
                               stages(i) = sum(p.stage(iter, :) == (i - 1));
                            end
                            c = clock;
                            disp(['iteration: ' num2str(iter) ' best overal fitness: ' num2str(bofit) ...
                                ' stages: ' mat2str(stages) ...
                                ' time: ' num2str(c(4)) ':' num2str(c(5)) ':' num2str(floor(c(6)))]);
                    end
                end
            end % iter
        end
        
        function init(p)
            p.nvars = length(p.posmin);
            p.pos = zeros(p.iterations, p.popsize, p.nvars);
            p.vel = zeros(p.iterations, p.popsize, p.nvars);
            p.fitness = zeros(p.iterations, p.popsize);
            p.stage = zeros(p.iterations, p.popsize);
            
            if isempty(p.initsize)
                p.initsize = p.popsize;
            end
            
            parfor i = 1:p.initsize
                [ipos(i,:), ivel(i,:), ifitness(i), istage(i)] = ...
                    p.initparticle;
            end
            
            sortin = p.sortp(ifitness, istage);
            bestin = sortin((end-p.popsize+1):end);
            bestin = bestin(randperm(p.popsize));
            
            p.pos(1,:,:) = ipos(bestin,:);
            p.vel(1,:,:) = ivel(bestin,:);
            p.fitness(1,:) = ifitness(bestin);
            p.stage(1,:) = istage(bestin);
            
            for iout = p.output
                switch iout{:}
                    case 'graphic'
                        p.plotparticles();
                    case 'text'
                        disp('simulation initialized')
                    case 'file'
                    otherwise
                        disp([iout{:} ' is not a valid output option']);
                end
            end
        end
        
        function [pnewpos pnewvel pnewfit pnewstage] = initparticle(p)
            succes = 0;
            while ~succes
                % create a new particle position
                pnewpos = p.initposmin + rand(1, p.nvars).*(p.initposmax-p.initposmin);
                % evaluate the fitness of the new paricle and check if this
                % is a valid position
                [errcode, pnewfit, pnewstage] = p.costfunc(pnewpos);
                if ~errcode
                    succes = 1;
                end
            end
            % create a new particle velocity
            vmin = -.01*(p.initposmax-p.initposmin);
            vmax = .01*(p.initposmax-p.initposmin);
            pnewvel = vmin + rand(1, p.nvars) .* (vmax-vmin);
        end
        
        function p = plotparticles(p)
            % PLOTPARTICLES makes a figure in which the data can be plotted
            p.fig = figure;
            p.axs = axes;
            hold on;
            p.sca = scatter(p.axs, 0, 0);
            for i = 1:p.popsize
                p.plt(i) = plot(p.axs, 0, 0);
            end
        end
        
        function p = updateplot(p, iter)
            % UPDATEPLOT updates the figure
            pscaled = p.scalepos;
            princ = princomp(squeeze(pscaled(iter,:,:)));
            ptrace = zeros(p.iterations, p.popsize, p.nvars);
            for j = 1:size(p.pos, 1)
                ptrace(j, :, :) = squeeze(p.pos(j, :, :)) * princ;
            end
            set(p.sca,'XData', ptrace(iter, :, 1), 'YData', ptrace(iter, :, 2));
            for i = 1:p.popsize
                set(p.plt(i),'XData', ptrace(1:iter, i, 1), 'YData', ptrace(1:iter, i, 2));
            end
            xlabel(['1st principle component: ' mat2str(princ(:,1), 3)])
            ylabel(['2nd principle component: ' mat2str(princ(:,2), 3)])
            drawnow
        end
        
        function pscaled = scalepos(p)
            pmin(1, 1, :) = p.posmin;
            pmax(1, 1, :) = p.posmax;
            pscaled = repmat(pmin, [p.iterations, p.popsize, 1]) ...
                + p.pos .* repmat(pmax - pmin, [p.iterations, p.popsize, 1]);
        end
        
        function sp = sortp(p, sortfit, sortstage)
            % SORTITER sorts an iteration
            bstage = max(sortstage);
            sp = [];
            for i = 1:(bstage+1)
                istage = find(sortstage == (i - 1));
                [~, si] = sort(sortfit(istage));
                sp = [sp istage(si)];
            end
        end
            
        function [bp bfit bstage] = bestpos(p, iter, part)
            % BESTPOS finds the best iteration of one particle
            bstage = max(p.stage(1:iter, part));
            maxi = find(p.stage(1:iter, part) == bstage);
            
            [bfit, bpi] = max(p.fitness(maxi, part));
            bp = p.pos(maxi(bpi), part, :);
        end
        
        function [blp blfit blstage] = localbestpos(p, iter, part)
            % BEST LOCALPOS finds the best iteration of the local particles
            % find the local particles
            [~, iLocal] = sort(p.relpos(part,:));
            
            blstage = max(max(p.stage(1:iter, iLocal(1:p.localsize))));
            [ai, bi] = find(p.stage(1:iter, iLocal(1:p.localsize)) == blstage);
            
            fittemp = p.fitness(1:iter, iLocal(1:p.localsize));
            [blfit, blfiti] = max(fittemp(p.stage(1:iter, iLocal(1:p.localsize)) == blstage));

            blp = p.pos(ai(blfiti), iLocal(bi(blfiti)), :);
        end
        
        function [bsp bsfit bsstage] = socialbestpos(p, iter, part)
            % BEST LOCALPOS finds the best iteration of the local particles
            % find the local particles
            iLocal = mod(part - 1 + (-p.socialsize : p.socialsize), p.popsize) + 1; % next neighbours

            bsstage = max(max(p.stage(1:iter, iLocal)));
            [ai, bi] = find(p.stage(1:iter, iLocal) == bsstage);
            
            fittemp = p.fitness(1:iter, iLocal);
            [bsfit, blfiti] = max(fittemp(p.stage(1:iter, iLocal) == bsstage));
            
            bsp = p.pos(ai(blfiti), iLocal(bi(blfiti)), :);
        end
        
        function [bop bofit bostage] = overalbestpos(p, iter)
            % OVERALBESTPOS finds the best iteration of all particles
            bostage = max(max(p.stage(1:iter, :)));
            [ai, bi] = find(p.stage(1:iter, :) == bostage);
            
            fittemp = p.fitness(1:iter, :);
            [bofit, blfiti] = max(fittemp(p.stage(1:iter, :) == bostage));
            bop = p.pos(ai(blfiti), bi(blfiti), :);
        end
                
        function relpos = calcrelpos(p, iter)
            ps_all = p.scalepos;
            pscaled = squeeze(ps_all(iter, :, :));
            relpos = zeros(p.popsize);
            for i = 1 : p.popsize
                for j = i+1 : p.popsize
                    relpos(i, j) = norm(pscaled(i, :) - pscaled(j, :));
                    relpos(j, i) = relpos(i, j);
                end
            end
        end
    end
    
end

