function fitness = walkcostfunction(mdl, varargin)
% WALKCOSTFUNCTION Calculates the fittness of a simulation
% IN: mdl (model object)
% OUT: fitness (struct)
% fitness.activation: average activation []
% fitness.avgtor: average torque [Nm]
% fitness.maxtor: maximal torque per joint, averaged over the joints [Nm]
% fitness.distance: distance traveled by the origin [m]
% fitness.time: duration of the simulation [s]
% fitness.speed: average speed of the subject [m/s]
% fitness.steplengthstd: standard deviation in the step length, normalized
% over the step length.

% simulation time
% the simulation is either terminated when an event is detected or is the
% maximal simulation time is reached
fitness.time = mdl.simt(end);

% data range
if nargin == 1
    % take the last nsteps of the simulation
    nSteps = 5;
    steps = [];
    for i = 2:length(mdl.simt)-(0.1/mdl.dt)
        if any(mdl.simrec(mdl.findcontrol('contactforce').recid([2 5]),i)) ...
                && any(mdl.simrec(mdl.findcontrol('contactforce').recid([2 5]),i+(0.1/mdl.dt))) ...
                && ~any(mdl.simrec(mdl.findcontrol('contactforce').recid([2 5]),i-1))
            steps = [steps i];
        end
    end
    if length(steps) > (nSteps + 1)
        range = steps(end-nSteps):steps(end);
    else
        range = 1:length(mdl.simt);
    end
else
    range = varargin{1};
end

% mass
diagM = diag(mdl.simcons.M);
M = sum(diagM(1:3:end));

% muscle activation
sid1 = length([mdl.x0; mdl.dx0(mdl.q_dep == 0)]); % first element of the controller
if ~isempty(mdl.findcontrol('reflex'))
    activations = mdl.simx(sid1 + mdl.findcontrol('reflex').sid(1:14), range); % muscle activations
    fitness.activation = sum(rms(activations,2))/14; % average activation
else
    fitness.activation = 0;
end

% muscle energy/work
if ~isempty(mdl.findcontrol('reflex'))
    [E, muscles] = muscleenergyexperditure(mdl);
    fitness.muscleenergy = mean(E.total(range(1:end-1)))/M;
    fitness.musclework  = mean([[muscles.m] [muscles.m]] * abs(E.mech(:, range(1:end-1))));
else
    fitness.muscleenergy = 0;
    fitness.musclework = 0;
end

% joint work/torque
torquecontrollers = {'reflex', 'ligaments', 'motion_generator', 'reverse_leg'};

ntor = 0;
for iC = 1:length(torquecontrollers)
    if ~isempty(mdl.findcontrol(torquecontrollers{iC}))
        ntor = ntor + 1;
        tor(ntor, :, :) = mdl.simrec(mdl.findcontrol(torquecontrollers{iC}).recid(end-5:end), range)/M;
    end
end

fitness.avgtor = sum(sum(abs(sum(tor,1)))) / numel(tor) * ntor;
fitness.maxtor = sum(max(abs(sum(tor,1)),[],3))/6;
fitness.maxtorhipknee = sum(max(abs(sum(tor(:,[1 2 4 5],:),1)),[],3))/4;
fitness.ligamentstor = sum(rms(squeeze(tor(2,:,:)),2)) / 6; % RMS of the ligaments torque


if ~isempty(mdl.findcontrol('reflex'))
    qid = [];
    qidp = [];
    for i = 1:length(mdl.findcontrol('reflex').motors)
        qid(end+1) = mdl.findcontrol('reflex').motors{i}.qid;
        qidp(end+1) = mdl.findcontrol('reflex').motors{i}.parent.qid(end);
    end
    relvel = mdl.simx(mdl.nq+qid,range) - mdl.simx(mdl.nq+qidp,range);
    fitness.work = sum(sum(relvel .* squeeze(sum(tor,1)))) ...
        / numel(range);
else
    fitness.work = 0;
end

% xped max joint torque
if ~isempty(mdl.findcontrol('xped'))
    xped = mdl.simrec(mdl.findcontrol('xped').recid, range);
    fitness.xped = max(max(abs(xped)));
else
    fitness.xped = [];
end

% average speed
[~, Tm_start] = Tmatrices(mdl.simx(:,range(1)), zeros(mdl.nq, 1), mdl.simcons.p_in);
[~, Tm_end] = Tmatrices(mdl.simx(:,range(end)), zeros(mdl.nq, 1), mdl.simcons.p_in);
distance = Tm_end(1) - Tm_start(1);
fitness.speed = distance/(mdl.simt(range(end))-mdl.simt(range(1)));

% step length
contact = mdl.findcontrol('contactforce');
heelleft = mdl.simrec(contact.recid(2),range);
heelright = mdl.simrec(contact.recid(contact.rec/2 + 2),range);
stepsleft = find(diff(heelleft > 0) > 0);
stepsright = find(diff(heelright > 0) > 0);
if length(stepsleft) > 3 && length(stepsright) > 3
    allsteps = [diff(stepsleft) diff(stepsright)];
    fitness.steplength = mean(allsteps) * mdl.dt;
else
    fitness.steplength = 0;
end

% step length deviation
% standard deviation in the step length, only calculated if at least 2
% steps are made. If less steps are made fitness is 1
if length(stepsleft) > 3 && length(stepsright) > 3
    allsteps = [diff(stepsleft) diff(stepsright)];
    fitness.steplengthstd = std(allsteps)/mean(allsteps);
else
    fitness.steplengthstd = 1;
end

end

